@@ -91,8 +91,9 @@ private:
     bool ReadPacketID (void);
     void L2_processing (void);
     int L2_quintuple_filter (void);
-    void Isoch_processing (void);
-    void UDPTriggers_processing (void);
+    void Isoch_processing (int);
+    bool InProgress(void);
+    Timestamp connect_done;
 
 #if HAVE_DECL_SO_TIMESTAMP
     // Structures needed for recvmsg
@@ -108,17 +109,6 @@ private:
     struct iphdr *ip_hdr;
     struct udphdr *udp_hdr;
 #endif
-#ifdef HAVE_UDPTRIGGERS
-    uint16_t packetidhash (int64_t);
-    struct fwtsf_entry_t {
-	bool free;
-	int64_t packetID;
-	uint32_t fwrxts1;
-	uint32_t fwrxts2;
-    };
-#define HASHTABLESIZE (1 << 16)
-    fwtsf_entry_t fwtsf_hashtable[HASHTABLESIZE];
-#endif
 }; // end class Server
 
 #endif // SERVER_H
